/***************************************************************************
 Copyright (C) 2007 Christoph Reichenbach <reichenb@colorado.edu>

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public Licence as
 published by the Free Software Foundaton; either version 2 of the
 Licence, or (at your option) any later version.

 It is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 merchantibility or fitness for a particular purpose. See the
 GNU General Public Licence for more details.

 You should have received a copy of the GNU General Public Licence
 along with this program; see the file COPYING. If not, write to
 the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.

***************************************************************************/

#include <cctype>
#include "adv-parser.hh"

AdvParser::AdvParser(void) {}

AdvParser::~AdvParser(void) {}

static direction
parse_dir(string dir_name)
{
	if (dir_name == "NORTH" || dir_name == "N")
		return NORTH;
	if (dir_name == "SOUTH" || dir_name == "S")
		return SOUTH;
	if (dir_name == "EAST" || dir_name == "E")
		return EAST;
	if (dir_name == "WEST" || dir_name == "W")
		return WEST;

	throw "Bad direction";
}

static void
to_upper(string &str)
{
	for (string::iterator i = str.begin(); i != str.end(); i++)
		*i = toupper(*i);
}

bool
AdvParser::parse(void)
{
	status();

	cout << "> ";
	string input;
	getline(cin, input);
	to_upper(input);

	string cmd_string = "";
	string param_string_buf = "";

	int offset = input.find_first_of(' ');

	if (offset == string::npos)
		cmd_string = input;
	else {
		cmd_string = input.substr(0, offset);
		param_string_buf = input.substr(offset + 1);
	}

	try {
		/* if this fails, we raise an exceptionn: x*/
		direction d = parse_dir(cmd_string);

		param_string_buf = cmd_string;
		cmd_string = "GO";
	} catch (char const* s) {
	}

	string *param_string = param_string_buf.empty() ? NULL : &param_string_buf;

	if (cmd_string == "QUIT"
	    || cmd_string == "EXIT") {
		quit();
		return false;
	}

	if (cmd_string == "GO") {
		try {
			direction dir = parse_dir(param_string_buf);

			go(dir);
		} catch (char const* s) {
			cout << "No such direction, ``" << param_string << "''\n";
			return true;
		}
	} else if (cmd_string == "LOOK" || cmd_string == "L")
		look(param_string);
	else if (cmd_string == "GET" || cmd_string == "T" || cmd_string == "TAKE")
		get(param_string);
	else if (cmd_string == "DROP")
		drop(param_string);
	else if (cmd_string == "USE" || cmd_string == "APPLY")
		use(param_string);
	else if (cmd_string == "INVENTORY" || cmd_string == "INV" || cmd_string == "I")
		inventory();
	else cout << "Sorry, but I have no idea what ``" + cmd_string + "'' means." << endl;

	return true;
}

void
AdvParser::status(void)
{
	cout << "This here should be a status message after you're done with 2.b..." << endl;
}

void
AdvParser::look(string *_)
{
	cout << "My eyes are missing!" << endl;
}

void
AdvParser::go(direction _)
{
	cout << "My legs! They're not implemented yet!" << endl;
}

void
AdvParser::get(string *_)
{
	cout << "Thanks, but I don't believe in ownership.  Not yet, anyway." << endl;
}

void
AdvParser::drop(string *_)
{
	cout << "I have no worldly possessions." << endl;
}

void
AdvParser::use(string *_)
{
	cout << "Use what, and how?" << endl;
}

void
AdvParser::inventory(void)
{
	cout << "What is this inventory you speak of?" << endl;
}

void
AdvParser::quit(void)
{
	cout << "Goodbye!" << endl;
}

